@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,18 +26,17 @@
 #define __EMPATHY_ACCOUNT_WIDGET_GENERIC_H__
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include <libmissioncontrol/mc-account.h>
 
 G_BEGIN_DECLS
 
 void       empathy_account_widget_handle_params     (McAccount   *account,
-						     GladeXML    *glade,
-						     const gchar *first_widget_name,
+						     GtkBuilder  *gui,
+						     const gchar *first_widget,
 						     ...);
 void       empathy_account_widget_add_forget_button (McAccount   *account,
-						     GladeXML    *glade,
+						     GtkBuilder  *gui,
 						     const gchar *button,
 						     const gchar *entry);
 GtkWidget *empathy_account_widget_generic_new       (McAccount   *account);
@@ -45,7 +44,9 @@ GtkWidget *empathy_account_widget_salut_new         (McAccount   *account);
 GtkWidget *empathy_account_widget_msn_new           (McAccount   *account);
 GtkWidget *empathy_account_widget_jabber_new        (McAccount   *account);
 GtkWidget *empathy_account_widget_icq_new           (McAccount   *account);
+GtkWidget *empathy_account_widget_aim_new           (McAccount   *account);
 GtkWidget *empathy_account_widget_yahoo_new         (McAccount   *account);
+GtkWidget *empathy_account_widget_groupwise_new     (McAccount   *account);
 
 G_END_DECLS
 
