@@ -26,7 +26,7 @@
 
 #include <gtk/gtkbutton.h>
 
-#include <libempathy/empathy-avatar.h>
+#include <libempathy/empathy-contact.h>
 
 G_BEGIN_DECLS
 
@@ -38,10 +38,12 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyAvatarChooser        EmpathyAvatarChooser;
 typedef struct _EmpathyAvatarChooserClass   EmpathyAvatarChooserClass;
-typedef struct _EmpathyAvatarChooserPrivate EmpathyAvatarChooserPrivate;
 
 struct _EmpathyAvatarChooser {
 	GtkButton parent;
+
+	/*<private>*/
+	gpointer priv;
 };
 
 struct _EmpathyAvatarChooserClass {
