@@ -47,10 +47,10 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyChat       EmpathyChat;
 typedef struct _EmpathyChatClass  EmpathyChatClass;
-typedef struct _EmpathyChatPriv   EmpathyChatPriv;
 
 struct _EmpathyChat {
 	GtkBin parent;
+	gpointer priv;
 
 	/* Protected */
 	EmpathyChatView *view;
@@ -71,17 +71,17 @@ const gchar *      empathy_chat_get_id               (EmpathyChat   *chat);
 const gchar *      empathy_chat_get_name             (EmpathyChat   *chat);
 const gchar *      empathy_chat_get_subject          (EmpathyChat   *chat);
 EmpathyContact *   empathy_chat_get_remote_contact   (EmpathyChat   *chat);
-guint              empathy_chat_get_members_count    (EmpathyChat   *chat);
+GtkWidget *        empathy_chat_get_contact_menu     (EmpathyChat   *chat);
 void               empathy_chat_clear                (EmpathyChat   *chat);
 void               empathy_chat_scroll_down          (EmpathyChat   *chat);
 void               empathy_chat_cut                  (EmpathyChat   *chat);
 void               empathy_chat_copy                 (EmpathyChat   *chat);
 void               empathy_chat_paste                (EmpathyChat   *chat);
 void               empathy_chat_correct_word         (EmpathyChat   *chat,
-						      GtkTextIter    start,
-						      GtkTextIter    end,
+						      GtkTextIter   *start,
+						      GtkTextIter   *end,
 						      const gchar   *new_word);
-
+gboolean           empathy_chat_is_room              (EmpathyChat   *chat);
 G_END_DECLS
 
 #endif /* __EMPATHY_CHAT_H__ */
