@@ -53,7 +53,8 @@ typedef enum {
 	EMPATHY_CONTACT_LIST_FEATURE_CONTACT_REMOVE = 1 << 3,
 	EMPATHY_CONTACT_LIST_FEATURE_CONTACT_DROP = 1 << 4,
 	EMPATHY_CONTACT_LIST_FEATURE_CONTACT_DRAG = 1 << 5,
-	EMPATHY_CONTACT_LIST_FEATURE_ALL = (1 << 6) - 1,
+	EMPATHY_CONTACT_LIST_FEATURE_CONTACT_TOOLTIP = 1 << 6,
+	EMPATHY_CONTACT_LIST_FEATURE_ALL = (1 << 7) - 1,
 } EmpathyContactListFeatureFlags;
 
 struct _EmpathyContactListView {
@@ -69,7 +70,7 @@ GType                      empathy_contact_list_view_get_type           (void) G
 EmpathyContactListView *   empathy_contact_list_view_new                (EmpathyContactListStore        *store,
 								         EmpathyContactListFeatureFlags  list_features,
 								         EmpathyContactFeatureFlags      contact_features);
-EmpathyContact *           empathy_contact_list_view_get_selected       (EmpathyContactListView         *view);
+EmpathyContact *           empathy_contact_list_view_dup_selected       (EmpathyContactListView         *view);
 gchar *                    empathy_contact_list_view_get_selected_group (EmpathyContactListView         *view);
 GtkWidget *                empathy_contact_list_view_get_contact_menu   (EmpathyContactListView         *view);
 GtkWidget *                empathy_contact_list_view_get_group_menu     (EmpathyContactListView         *view);
