@@ -25,13 +25,13 @@
 #include <stdlib.h>
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <glib/gi18n-lib.h>
 
 #include <libmissioncontrol/mc-account.h>
 #include <libmissioncontrol/mission-control.h>
 
-#include <libempathy/empathy-contact-factory.h>
+#include <libempathy/empathy-call-factory.h>
+#include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-utils.h>
@@ -44,6 +44,16 @@
 #include "empathy-new-message-dialog.h"
 #include "empathy-account-chooser.h"
 
+/**
+ * SECTION:empathy-new-message-dialog
+ * @title: EmpathyNewMessageDialog
+ * @short_description: A dialog to show a new message
+ * @include: libempathy-gtk/empathy-new-message-dialog.h
+ *
+ * #EmpathyNewMessageDialog is a dialog which allows a text chat or
+ * call to be started with any contact on any enabled account.
+ */
+
 typedef struct {
 	GtkWidget *dialog;
 	GtkWidget *table_contact;
@@ -65,48 +75,54 @@ new_message_dialog_account_changed_cb (GtkWidget               *widget,
 				       EmpathyNewMessageDialog *dialog)
 {
 	EmpathyAccountChooser *chooser;
-	McAccount            *account;
+	TpConnection          *connection;
 	EmpathyTpContactList *contact_list;
-	GList                *members, *l;
+	GList                *members;
 	GtkListStore         *store;
 	GtkEntryCompletion   *completion;
 	GtkTreeIter           iter;
 	gchar                *tmpstr;
 
-	chooser = EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser);
-	account = empathy_account_chooser_get_account (chooser);
-	contact_list = empathy_contact_manager_get_list (dialog->contact_manager,
-							 account);
-	members = empathy_contact_list_get_members (EMPATHY_CONTACT_LIST (contact_list));
+	/* Remove completions */
 	completion = gtk_entry_get_completion (GTK_ENTRY (dialog->entry_id));
 	store = GTK_LIST_STORE (gtk_entry_completion_get_model (completion));
 	gtk_list_store_clear (store);
 
-	for (l = members; l; l = l->next) {
-		EmpathyContact *contact = l->data;
+	/* Get members of the new account */
+	chooser = EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser);
+	connection = empathy_account_chooser_get_connection (chooser);
+	if (!connection) {
+		return;
+	}
+	contact_list = empathy_contact_manager_get_list (dialog->contact_manager,
+							 connection);
+	members = empathy_contact_list_get_members (EMPATHY_CONTACT_LIST (contact_list));
 
-		if (!empathy_contact_is_online (contact)) {
-			continue;
+	/* Add members to the completion */
+	while (members) {
+		EmpathyContact *contact = members->data;
+
+		if (empathy_contact_is_online (contact)) {
+			DEBUG ("Adding contact ID %s, Name %s",
+			       empathy_contact_get_id (contact),
+			       empathy_contact_get_name (contact));
+
+			tmpstr = g_strdup_printf ("%s (%s)",
+				empathy_contact_get_name (contact),
+				empathy_contact_get_id (contact));
+
+			gtk_list_store_insert_with_values (store, &iter, -1,
+				COMPLETION_COL_TEXT, tmpstr,
+				COMPLETION_COL_ID, empathy_contact_get_id (contact),
+				COMPLETION_COL_NAME, empathy_contact_get_name (contact),
+				-1);
+
+			g_free (tmpstr);
 		}
 
-		DEBUG ("Adding contact ID %s, Name %s",
-		       empathy_contact_get_id (contact),
-		       empathy_contact_get_name (contact));
-
-		tmpstr = g_strdup_printf ("%s (%s)",
-					  empathy_contact_get_name (contact),
-					  empathy_contact_get_id (contact));
-
-		gtk_list_store_insert_with_values (store, &iter, -1,
-			COMPLETION_COL_TEXT, tmpstr,
-			COMPLETION_COL_ID, empathy_contact_get_id (contact),
-			COMPLETION_COL_NAME, empathy_contact_get_name (contact),
-			-1);
-
-		g_free (tmpstr);
+		g_object_unref (contact);
+		members = g_list_delete_link (members, members);
 	}
-
-	g_object_unref (account);
 }
 
 static gboolean
@@ -165,32 +181,52 @@ new_message_dialog_match_func (GtkEntryCompletion *completion,
 	return FALSE;
 }
 
+static void
+new_message_dialog_call_got_contact_cb (EmpathyTpContactFactory *factory,
+					EmpathyContact          *contact,
+					const GError            *error,
+					gpointer                 user_data,
+					GObject                 *weak_object)
+{
+	EmpathyCallFactory *call_factory;
+
+	if (error != NULL) {
+		DEBUG ("Error: %s", error->message);
+		return;
+	}
+
+	call_factory = empathy_call_factory_get();
+	empathy_call_factory_new_call (call_factory, contact);
+}
+
 static void
 new_message_dialog_response_cb (GtkWidget               *widget,
 				gint                    response,
 				EmpathyNewMessageDialog *dialog)
 {
-	McAccount   *account;
+	TpConnection *connection;
 	const gchar *id;
 
-	account = empathy_account_chooser_get_account (EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser));
+	connection = empathy_account_chooser_get_connection (
+		EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser));
 	id = gtk_entry_get_text (GTK_ENTRY (dialog->entry_id));
-	if (!account || G_STR_EMPTY (id)) {
-		if (account) {
-			g_object_unref (account);
-		}
+	if (!connection || EMP_STR_EMPTY (id)) {
 		gtk_widget_destroy (widget);
 		return;
 	}
 
 	if (response == 1) {
-		empathy_dispatcher_call_with_contact_id (account, id);
-	}
-	else if (response == 2) {
-		empathy_dispatcher_chat_with_contact_id (account, id);
+		EmpathyTpContactFactory *factory;
+
+		factory = empathy_tp_contact_factory_dup_singleton (connection);
+		empathy_tp_contact_factory_get_from_id (factory, id,
+			new_message_dialog_call_got_contact_cb,
+			NULL, NULL, NULL);
+		g_object_unref (factory);
+	} else if (response == 2) {
+		empathy_dispatcher_chat_with_contact_id (connection, id, NULL, NULL);
 	}
 
-	g_object_unref (account);
 	gtk_widget_destroy (widget);
 }
 
@@ -202,7 +238,7 @@ new_message_change_state_button_cb  (GtkEditable             *editable,
 	gboolean     sensitive;
 
 	id = gtk_entry_get_text (GTK_ENTRY (editable));
-	sensitive = !G_STR_EMPTY (id);
+	sensitive = !EMP_STR_EMPTY (id);
 	
 	gtk_widget_set_sensitive (dialog->button_chat, sensitive);
 	gtk_widget_set_sensitive (dialog->button_call, sensitive);
@@ -216,11 +252,19 @@ new_message_dialog_destroy_cb (GtkWidget               *widget,
 	g_free (dialog);
 }
 
+/**
+ * empathy_new_message_dialog_show:
+ * @parent: parent #GtkWindow of the dialog
+ *
+ * Create a new #EmpathyNewMessageDialog and show it.
+ *
+ * Return value: the new #EmpathyNewMessageDialog
+ */
 GtkWidget *
 empathy_new_message_dialog_show (GtkWindow *parent)
 {
 	static EmpathyNewMessageDialog *dialog = NULL;
-	GladeXML                       *glade;
+	GtkBuilder                     *gui;
 	gchar                          *filename;
 	GtkEntryCompletion             *completion;
 	GtkListStore                   *model;
@@ -233,13 +277,11 @@ empathy_new_message_dialog_show (GtkWindow *parent)
 	dialog = g_new0 (EmpathyNewMessageDialog, 1);
 
 	/* create a contact manager */
-	dialog->contact_manager = empathy_contact_manager_new ();
+	dialog->contact_manager = empathy_contact_manager_dup_singleton ();
 
-	filename = empathy_file_lookup ("empathy-new-message-dialog.glade",
+	filename = empathy_file_lookup ("empathy-new-message-dialog.ui",
 					"libempathy-gtk");
-	glade = empathy_glade_get_file (filename,
-				        "new_message_dialog",
-				        NULL,
+	gui = empathy_builder_get_file (filename,
 				        "new_message_dialog", &dialog->dialog,
 				        "table_contact", &dialog->table_contact,
 				        "entry_id", &dialog->entry_id,
@@ -263,7 +305,7 @@ empathy_new_message_dialog_show (GtkWindow *parent)
 	g_object_unref(completion);
 	g_object_unref(model);
 
-	empathy_glade_connect (glade, dialog,
+	empathy_builder_connect (gui, dialog,
 			       "new_message_dialog", "destroy", new_message_dialog_destroy_cb,
 			       "new_message_dialog", "response", new_message_dialog_response_cb,
 			       "entry_id", "changed", new_message_change_state_button_cb,
@@ -271,7 +313,7 @@ empathy_new_message_dialog_show (GtkWindow *parent)
 
 	g_object_add_weak_pointer (G_OBJECT (dialog->dialog), (gpointer) &dialog);
 
-	g_object_unref (glade);
+	g_object_unref (gui);
 
 	/* Create account chooser */
 	dialog->account_chooser = empathy_account_chooser_new ();
@@ -300,4 +342,3 @@ empathy_new_message_dialog_show (GtkWindow *parent)
 
 	return dialog->dialog;
 }
-
