@@ -37,6 +37,10 @@ typedef struct {
 	gboolean   auto_connect;
 	gboolean favorite;
 	EmpathyTpChat *tp_chat;
+	gchar *subject;
+	guint members_count;
+	gboolean invite_only;
+	gboolean need_password;
 } EmpathyChatroomPriv;
 
 
@@ -58,6 +62,10 @@ enum {
 	PROP_AUTO_CONNECT,
 	PROP_FAVORITE,
 	PROP_TP_CHAT,
+	PROP_SUBJECT,
+	PROP_MEMBERS_COUNT,
+	PROP_NEED_PASSWORD,
+	PROP_INVITE_ONLY
 };
 
 G_DEFINE_TYPE (EmpathyChatroom, empathy_chatroom, G_TYPE_OBJECT);
@@ -103,17 +111,14 @@ empathy_chatroom_class_init (EmpathyChatroomClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
-  g_object_class_install_property (object_class,
-      PROP_FAVORITE,
-      g_param_spec_boolean ("favorite",
-        "Favorite",
-        "TRUE if the chatroom is in user's favorite list",
-        FALSE,
-        G_PARAM_READWRITE |
-        G_PARAM_CONSTRUCT |
-        G_PARAM_STATIC_NAME |
-        G_PARAM_STATIC_NICK |
-        G_PARAM_STATIC_BLURB));
+	g_object_class_install_property (object_class,
+					 PROP_FAVORITE,
+					 g_param_spec_boolean ("favorite",
+							       "Favorite",
+							       "TRUE if the chatroom is in user's favorite list",
+							       FALSE,
+							       G_PARAM_READWRITE |
+							       G_PARAM_CONSTRUCT));
 
 	g_object_class_install_property (object_class,
 					 PROP_TP_CHAT,
@@ -123,6 +128,56 @@ empathy_chatroom_class_init (EmpathyChatroomClass *klass)
 							      EMPATHY_TYPE_TP_CHAT,
 							      G_PARAM_READWRITE));
 
+  g_object_class_install_property (object_class,
+      PROP_SUBJECT,
+      g_param_spec_string ("subject",
+        "Subject",
+        "The chatroom's subject",
+        "",
+        G_PARAM_READWRITE |
+        G_PARAM_CONSTRUCT |
+        G_PARAM_STATIC_NAME |
+        G_PARAM_STATIC_NICK |
+        G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+      PROP_MEMBERS_COUNT,
+      g_param_spec_uint ("members-count",
+        "Members count",
+        "The chatroom's members count",
+        0,
+        G_MAXUINT,
+        0,
+        G_PARAM_READWRITE |
+        G_PARAM_CONSTRUCT |
+        G_PARAM_STATIC_NAME |
+        G_PARAM_STATIC_NICK |
+        G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+      PROP_INVITE_ONLY,
+      g_param_spec_boolean ("invite-only",
+        "Invite Only",
+        "The chatroom is invite only",
+        FALSE,
+        G_PARAM_READWRITE |
+        G_PARAM_CONSTRUCT |
+        G_PARAM_STATIC_NAME |
+        G_PARAM_STATIC_NICK |
+        G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+      PROP_NEED_PASSWORD,
+      g_param_spec_boolean ("need-password",
+        "Password Needed",
+        "The chatroom is password protected",
+        FALSE,
+        G_PARAM_READWRITE |
+        G_PARAM_CONSTRUCT |
+        G_PARAM_STATIC_NAME |
+        G_PARAM_STATIC_NICK |
+        G_PARAM_STATIC_BLURB));
+
 	g_type_class_add_private (object_class, sizeof (EmpathyChatroomPriv));
 }
 
@@ -175,12 +230,24 @@ chatroom_get_property (GObject    *object,
 	case PROP_AUTO_CONNECT:
 		g_value_set_boolean (value, priv->auto_connect);
 		break;
-  case PROP_FAVORITE:
-    g_value_set_boolean (value, priv->favorite);
-    break;
+	case PROP_FAVORITE:
+		g_value_set_boolean (value, priv->favorite);
+		break;
 	case PROP_TP_CHAT:
 		g_value_set_object (value, priv->tp_chat);
 		break;
+  case PROP_SUBJECT:
+    g_value_set_string (value, priv->subject);
+    break;
+  case PROP_MEMBERS_COUNT:
+    g_value_set_uint (value, priv->members_count);
+    break;
+  case PROP_INVITE_ONLY:
+    g_value_set_boolean (value, priv->invite_only);
+    break;
+  case PROP_NEED_PASSWORD:
+    g_value_set_boolean (value, priv->need_password);
+    break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -214,30 +281,30 @@ chatroom_set_property (GObject      *object,
 		empathy_chatroom_set_auto_connect (EMPATHY_CHATROOM (object),
 						  g_value_get_boolean (value));
 		break;
-  case PROP_FAVORITE:
-    priv->favorite = g_value_get_boolean (value);
-    if (!priv->favorite)
-      {
-        empathy_chatroom_set_auto_connect (EMPATHY_CHATROOM (object),
-            FALSE);
-      }
-    break;
-	case PROP_TP_CHAT: {
-		GObject *chat = g_value_dup_object (value);
-
-		if (chat == (GObject *) priv->tp_chat)
-			break;
-
-		g_assert (chat == NULL || priv->tp_chat == NULL);
-
-		if (priv->tp_chat != NULL) {
-			g_object_unref (priv->tp_chat);
-			priv->tp_chat = NULL;
-		} else {
-			priv->tp_chat = EMPATHY_TP_CHAT (chat);
-		}
+	case PROP_FAVORITE:
+		empathy_chatroom_set_favorite (EMPATHY_CHATROOM (object),
+					       g_value_get_boolean (value));
 		break;
-	}
+	case PROP_TP_CHAT:
+		empathy_chatroom_set_tp_chat (EMPATHY_CHATROOM (object),
+					      g_value_get_object (value));
+		break;
+  case PROP_SUBJECT:
+    empathy_chatroom_set_subject (EMPATHY_CHATROOM (object),
+        g_value_get_string (value));
+    break;
+  case PROP_MEMBERS_COUNT:
+    empathy_chatroom_set_members_count (EMPATHY_CHATROOM (object),
+        g_value_get_uint (value));
+    break;
+  case PROP_NEED_PASSWORD:
+    empathy_chatroom_set_need_password (EMPATHY_CHATROOM (object),
+        g_value_get_boolean (value));
+    break;
+  case PROP_INVITE_ONLY:
+    empathy_chatroom_set_invite_only (EMPATHY_CHATROOM (object),
+        g_value_get_boolean (value));
+    break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -390,12 +457,11 @@ empathy_chatroom_set_auto_connect (EmpathyChatroom *chatroom,
 
 	priv->auto_connect = auto_connect;
 
-  if (priv->auto_connect)
-    {
-      /* auto_connect implies favorite */
-      priv->favorite = TRUE;
-      g_object_notify (G_OBJECT (chatroom), "favorite");
-    }
+	if (priv->auto_connect) {
+		/* auto_connect implies favorite */
+		priv->favorite = TRUE;
+		g_object_notify (G_OBJECT (chatroom), "favorite");
+	}
 
 	g_object_notify (G_OBJECT (chatroom), "auto-connect");
 }
@@ -418,12 +484,13 @@ empathy_chatroom_equal (gconstpointer v1,
 	room_a = empathy_chatroom_get_room (EMPATHY_CHATROOM (v1));
 	room_b = empathy_chatroom_get_room (EMPATHY_CHATROOM (v2));
 
-	return empathy_account_equal (account_a, account_b) && !tp_strdiff (room_a,
-      room_b);
+	return empathy_account_equal (account_a, account_b) &&
+	       !tp_strdiff (room_a, room_b);
 }
 
 EmpathyTpChat *
-empathy_chatroom_get_tp_chat (EmpathyChatroom *chatroom) {
+empathy_chatroom_get_tp_chat (EmpathyChatroom *chatroom)
+{
 	EmpathyChatroomPriv *priv;
 
 	g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), NULL);
@@ -432,3 +499,172 @@ empathy_chatroom_get_tp_chat (EmpathyChatroom *chatroom) {
 
 	return priv->tp_chat;
 }
+
+const gchar *
+empathy_chatroom_get_subject (EmpathyChatroom *chatroom)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), NULL);
+
+  priv = GET_PRIV (chatroom);
+
+  return priv->subject;
+}
+
+void
+empathy_chatroom_set_subject (EmpathyChatroom *chatroom,
+                              const gchar *subject)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_CHATROOM (chatroom));
+
+  priv = GET_PRIV (chatroom);
+
+  g_free (priv->subject);
+  priv->subject = NULL;
+
+  if (subject)
+    priv->subject = g_strdup (subject);
+
+  g_object_notify (G_OBJECT (chatroom), "subject");
+}
+
+guint
+empathy_chatroom_get_members_count (EmpathyChatroom *chatroom)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), 0);
+
+  priv = GET_PRIV (chatroom);
+
+  return priv->members_count;
+}
+
+void
+empathy_chatroom_set_members_count (EmpathyChatroom *chatroom,
+                                    guint count)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_CHATROOM (chatroom));
+
+  priv = GET_PRIV (chatroom);
+
+  priv->members_count = count;
+
+  g_object_notify (G_OBJECT (chatroom), "members-count");
+}
+
+gboolean
+empathy_chatroom_get_need_password (EmpathyChatroom *chatroom)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), FALSE);
+
+  priv = GET_PRIV (chatroom);
+
+  return priv->need_password;
+}
+
+void
+empathy_chatroom_set_need_password (EmpathyChatroom *chatroom,
+                                    gboolean need_password)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_CHATROOM (chatroom));
+
+  priv = GET_PRIV (chatroom);
+
+  priv->need_password = need_password;
+
+  g_object_notify (G_OBJECT (chatroom), "need-password");
+}
+
+gboolean
+empathy_chatroom_get_invite_only (EmpathyChatroom *chatroom)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), FALSE);
+
+  priv = GET_PRIV (chatroom);
+
+  return priv->invite_only;
+}
+
+void
+empathy_chatroom_set_invite_only (EmpathyChatroom *chatroom,
+                                  gboolean invite_only)
+{
+  EmpathyChatroomPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_CHATROOM (chatroom));
+
+  priv = GET_PRIV (chatroom);
+
+  priv->invite_only = invite_only;
+
+  g_object_notify (G_OBJECT (chatroom), "invite-only");
+}
+
+void
+empathy_chatroom_set_tp_chat (EmpathyChatroom *chatroom,
+			      EmpathyTpChat   *tp_chat)
+{
+	EmpathyChatroomPriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_CHATROOM (chatroom));
+	g_return_if_fail (tp_chat == NULL || EMPATHY_IS_TP_CHAT (tp_chat));
+
+	priv = GET_PRIV (chatroom);
+
+	if (priv->tp_chat == tp_chat) {
+		return;
+	}
+
+	if (priv->tp_chat != NULL) {
+		g_object_unref (priv->tp_chat);
+	}
+
+	priv->tp_chat = tp_chat ? g_object_ref (tp_chat) : NULL;
+	g_object_notify (G_OBJECT (chatroom), "tp-chat");
+}
+
+gboolean
+empathy_chatroom_is_favorite (EmpathyChatroom *chatroom)
+{
+	EmpathyChatroomPriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_CHATROOM (chatroom), FALSE);
+
+	priv = GET_PRIV (chatroom);
+
+	return priv->favorite;
+}
+
+void
+empathy_chatroom_set_favorite (EmpathyChatroom *chatroom,
+			       gboolean         favorite)
+{
+	EmpathyChatroomPriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_CHATROOM (chatroom));
+
+	priv = GET_PRIV (chatroom);
+
+	if (priv->favorite == favorite) {
+		return;
+	}
+
+	priv->favorite = favorite;
+	if (!priv->favorite) {
+		empathy_chatroom_set_auto_connect (chatroom, FALSE);
+	}
+	g_object_notify (G_OBJECT (chatroom), "favorite");
+}
+
