@@ -42,10 +42,10 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyTpChat      EmpathyTpChat;
 typedef struct _EmpathyTpChatClass EmpathyTpChatClass;
-typedef struct _EmpathyTpChatPriv  EmpathyTpChatPriv;
 
 struct _EmpathyTpChat {
-	GObject      parent;
+	GObject parent;
+	gpointer priv;
 };
 
 struct _EmpathyTpChatClass {
@@ -53,16 +53,13 @@ struct _EmpathyTpChatClass {
 };
 
 GType          empathy_tp_chat_get_type             (void) G_GNUC_CONST;
-EmpathyTpChat *empathy_tp_chat_new                  (TpChannel          *channel,
-						     gboolean            acknowledge);
+EmpathyTpChat *empathy_tp_chat_new                  (TpChannel          *channel);
+void           empathy_tp_chat_close                (EmpathyTpChat      *chat);
 const gchar *  empathy_tp_chat_get_id               (EmpathyTpChat      *chat);
 EmpathyContact*empathy_tp_chat_get_remote_contact   (EmpathyTpChat      *chat);
-McAccount *    empathy_tp_chat_get_account          (EmpathyTpChat      *chat);
+TpChannel *    empathy_tp_chat_get_channel          (EmpathyTpChat      *chat);
+TpConnection * empathy_tp_chat_get_connection       (EmpathyTpChat      *chat);
 gboolean       empathy_tp_chat_is_ready             (EmpathyTpChat      *chat);
-guint          empathy_tp_chat_get_members_count    (EmpathyTpChat      *chat);
-void           empathy_tp_chat_set_acknowledge      (EmpathyTpChat      *chat,
-						     gboolean            acknowledge);
-void           empathy_tp_chat_emit_pendings        (EmpathyTpChat      *chat);
 void           empathy_tp_chat_send                 (EmpathyTpChat      *chat,
 						     EmpathyMessage     *message);
 void           empathy_tp_chat_set_state            (EmpathyTpChat      *chat,
@@ -71,6 +68,13 @@ void           empathy_tp_chat_set_property         (EmpathyTpChat      *chat,
 						     const gchar        *name,
 						     const GValue       *value);
 
+/* Returns a read-only list of pending messages (should be a copy maybe ?) */
+const GList *  empathy_tp_chat_get_pending_messages (EmpathyTpChat *chat);
+void           empathy_tp_chat_acknowledge_message (EmpathyTpChat *chat,
+						     EmpathyMessage *message);
+void           empathy_tp_chat_acknowledge_messages (EmpathyTpChat *chat,
+						     const GList *messages);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_TP_CHAT_H__ */
