@@ -37,12 +37,18 @@
 #include <telepathy-glib/dbus.h>
 
 #include "empathy-utils.h"
-#include "empathy-contact-factory.h"
 #include "empathy-contact-manager.h"
+#include "empathy-dispatcher.h"
+#include "empathy-dispatch-operation.h"
+#include "empathy-idle.h"
+#include "empathy-tp-call.h"
+
+#include <extensions/extensions.h>
 
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include "empathy-debug.h"
 
+
 void
 empathy_init (void)
 {
@@ -64,6 +70,8 @@ empathy_init (void)
 	empathy_debug_set_flags (g_getenv ("EMPATHY_DEBUG"));
 	tp_debug_divert_messages (g_getenv ("EMPATHY_LOGFILE"));
 
+	emp_cli_init ();
+
 	initialized = TRUE;
 }
 
@@ -221,7 +229,7 @@ empathy_account_equal (gconstpointer a,
 }
 
 MissionControl *
-empathy_mission_control_new (void)
+empathy_mission_control_dup_singleton (void)
 {
 	static MissionControl *mc = NULL;
 
@@ -323,97 +331,6 @@ empathy_file_lookup (const gchar *filename, const gchar *subdir)
 	return path;
 }
 
-typedef struct {
-	EmpathyRunUntilReadyFunc  func;
-	gpointer                  user_data;
-	GObject                  *object;
-	GMainLoop                *loop;
-} RunUntilReadyData;
-
-static void
-run_until_ready_cb (RunUntilReadyData *data)
-{
-	if (!data->func || data->func (data->object, data->user_data)) {
-		DEBUG ("Object %p is ready", data->object);
-		g_main_loop_quit (data->loop);
-	}
-}
-
-static gboolean
-object_is_ready (GObject *object,
-		 gpointer user_data)
-{
-	gboolean ready;
-
-	g_object_get (object, "ready", &ready, NULL);
-
-	return ready;
-}
-
-void
-empathy_run_until_ready_full (gpointer                  object,
-			      const gchar              *signal,
-			      EmpathyRunUntilReadyFunc  func,
-			      gpointer                  user_data,
-			      GMainLoop               **loop)
-{
-	RunUntilReadyData  data;
-	gulong             signal_id;
-
-	g_return_if_fail (G_IS_OBJECT (object));
-	g_return_if_fail (signal != NULL);
-
-	if (func && func (object, user_data)) {
-		return;
-	}
-
-	DEBUG ("Starting run until ready for object %p", object);
-
-	data.func = func;
-	data.user_data = user_data;
-	data.object = object;
-	data.loop = g_main_loop_new (NULL, FALSE);
-
-	signal_id = g_signal_connect_swapped (object, signal,
-					      G_CALLBACK (run_until_ready_cb),
-					      &data);
-	if (loop != NULL) {
-		*loop = data.loop;
-	}
-
-	g_main_loop_run (data.loop);
-
-	if (loop != NULL) {
-		*loop = NULL;
-	}
-
-	g_signal_handler_disconnect (object, signal_id);
-	g_main_loop_unref (data.loop);
-}
-
-void
-empathy_run_until_ready (gpointer object)
-{
-	empathy_run_until_ready_full (object, "notify::ready", object_is_ready,
-				      NULL, NULL);
-}
-
-McAccount *
-empathy_channel_get_account (TpChannel *channel)
-{
-	TpConnection   *connection;
-	McAccount      *account;
-	MissionControl *mc;
-
-	g_object_get (channel, "connection", &connection, NULL);
-	mc = empathy_mission_control_new ();
-	account = mission_control_get_account_for_tpconnection (mc, connection, NULL);
-	g_object_unref (connection);
-	g_object_unref (mc);
-
-	return account;
-}
-
 guint
 empathy_proxy_hash (gconstpointer key)
 {
@@ -444,121 +361,21 @@ empathy_proxy_equal (gconstpointer a,
 	       g_str_equal (proxy_a->bus_name, proxy_b->bus_name);
 }
 
-typedef struct {
-	gint timeout_ms;
-	gchar *channel_type;
-	guint handle_type;
-	empathy_connection_callback_for_request_channel callback;
-	gpointer user_data;
-	GDestroyNotify destroy;
-	TpHandle handle;
-	gboolean suppress_handler;
-	guint ref_count;
-} ConnectionRequestChannelData;
-
-static void
-connection_request_channel_data_unref (gpointer user_data)
+gboolean
+empathy_check_available_state (void)
 {
-	ConnectionRequestChannelData *data = (ConnectionRequestChannelData*) user_data;
+	McPresence presence;
+	EmpathyIdle *idle;
 
-	if (--data->ref_count == 0) {
-		g_free (data->channel_type);
-		if (data->destroy) {
-			data->destroy (data->user_data);
-		}
-		g_slice_free (ConnectionRequestChannelData, data);
-	}
-}
+	idle = empathy_idle_dup_singleton ();
+	presence = empathy_idle_get_state (idle);
+	g_object_unref (idle);
 
-static void
-connection_request_channel_cb (TpConnection *connection,
-			       const gchar *object_path,
-			       const GError *error,
-			       gpointer user_data,
-			       GObject *weak_object)
-{
-	ConnectionRequestChannelData *data = (ConnectionRequestChannelData*) user_data;
-	TpChannel *channel;
-
-	if (!data->callback) {
-		return;
+	if (presence != MC_PRESENCE_AVAILABLE &&
+		presence != MC_PRESENCE_UNSET) {
+		return FALSE;    
 	}
 
-	if (error) {
-		data->callback (connection, NULL, error, data->user_data, weak_object);
-		return;
-	}
-
-	channel = tp_channel_new (connection, object_path,
-				  data->channel_type,
-				  data->handle_type,
-				  data->handle, NULL);
-
-	data->callback (connection, channel, NULL, data->user_data, weak_object);
-	g_object_unref (channel);
-}
-
-static void
-connection_request_handles_cb (TpConnection *connection,
-			       const GArray *handles,
-			       const GError *error,
-			       gpointer user_data,
-			       GObject *weak_object)
-{
-	ConnectionRequestChannelData *data = (ConnectionRequestChannelData*) user_data;
-
-	if (error) {
-		if (data->callback) {
-			data->callback (connection, NULL, error, data->user_data, weak_object);
-		}
-		return;
-	}
-
-	data->handle = g_array_index (handles, guint, 0);
-	data->ref_count++;
-	tp_cli_connection_call_request_channel (connection, data->timeout_ms,
-						data->channel_type,
-						data->handle_type,
-						data->handle,
-						data->suppress_handler,
-						connection_request_channel_cb,
-						data,
-						(GDestroyNotify) connection_request_channel_data_unref,
-						weak_object);
-}
-
-void
-empathy_connection_request_channel (TpConnection *connection,
-				    gint timeout_ms,
-				    const gchar *channel_type,
-				    guint handle_type,
-				    const gchar *name,
-				    gboolean suppress_handler,
-				    empathy_connection_callback_for_request_channel callback,
-				    gpointer user_data,
-				    GDestroyNotify destroy,
-				    GObject *weak_object)
-{
-	const gchar *names[] = {name, NULL};
-	ConnectionRequestChannelData *data;
-
-	data = g_slice_new (ConnectionRequestChannelData);
-	data->timeout_ms = timeout_ms;
-	data->channel_type = g_strdup (channel_type);
-	data->handle_type = handle_type;
-	data->callback = callback;
-	data->user_data = user_data;
-	data->destroy = destroy;
-	data->handle = 0;
-	data->suppress_handler = suppress_handler;
-	data->ref_count = 1;
-	tp_cli_connection_call_request_handles (connection,
-						timeout_ms,
-						handle_type,
-						names,
-						connection_request_handles_cb,
-						data,
-						(GDestroyNotify) connection_request_channel_data_unref,
-						weak_object);
+	return TRUE;
 }
 
