@@ -14,9 +14,9 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
@@ -42,8 +42,10 @@
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-chatroom-manager.h>
+#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-dispatch-operation.h>
+#include <libempathy/empathy-log-manager.h>
 #include <libempathy/empathy-tp-chat.h>
 #include <libempathy/empathy-tp-call.h>
 
@@ -95,10 +97,16 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
 		}
 
 		if (id) {
+			EmpathyAccountManager *manager;
+			TpConnection *connection;
 			McAccount *account;
 
-			account = empathy_tp_chat_get_account (tp_chat);
+			manager = empathy_account_manager_dup_singleton ();
+			connection = empathy_tp_chat_get_connection (tp_chat);
+			account = empathy_account_manager_get_account (manager,
+								       connection);
 			chat = empathy_chat_window_find_chat (account, id);
+			g_object_unref (manager);
 		}
 
 		if (chat) {
@@ -124,6 +132,7 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
 			empathy_dispatch_operation_get_channel_wrapper (operation));
 		empathy_ft_manager_add_tp_file (ft_manager, tp_file);
 		empathy_dispatch_operation_claim (operation);
+		g_object_unref (ft_manager);
 	}
 }
 
@@ -220,7 +229,7 @@ create_salut_account (void)
 	GError     *error = NULL;
 
 	/* Check if we already created a salut account */
-	empathy_conf_get_bool (empathy_conf_get(),
+	empathy_conf_get_bool (empathy_conf_get (),
 			       EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
 			       &salut_created);
 	if (salut_created) {
@@ -303,7 +312,7 @@ create_salut_account (void)
 /* The code that handles single-instance and startup notification is
  * copied from gedit.
  *
- * Copyright (C) 2005 - Paolo Maggi 
+ * Copyright (C) 2005 - Paolo Maggi
  */
 static void
 on_bacon_message_received (const char *message,
@@ -410,13 +419,15 @@ main (int argc, char *argv[])
 	guint32            startup_timestamp;
 	EmpathyStatusIcon *icon;
 	EmpathyDispatcher *dispatcher;
+	EmpathyLogManager *log_manager;
 	EmpathyChatroomManager *chatroom_manager;
+	EmpathyFTManager  *ft_manager;
 	EmpathyCallFactory *call_factory;
 	GtkWidget         *window;
 	MissionControl    *mc;
 	EmpathyIdle       *idle;
 	gboolean           autoconnect = TRUE;
-	gboolean           no_connect = FALSE; 
+	gboolean           no_connect = FALSE;
 	gboolean           hide_contact_list = FALSE;
 	gboolean           accounts_dialog = FALSE;
 	GError            *error = NULL;
@@ -451,6 +462,7 @@ main (int argc, char *argv[])
 
 	empathy_gtk_init ();
 	g_set_application_name (_(PACKAGE_NAME));
+	g_setenv ("PULSE_PROP_media.role", "phone", TRUE);
 
 	gst_init (&argc, &argv);
 
@@ -521,7 +533,7 @@ main (int argc, char *argv[])
 				 use_nm_notify_cb, idle);
 
 	/* Autoconnect */
-	empathy_conf_get_bool (empathy_conf_get(),
+	empathy_conf_get_bool (empathy_conf_get (),
 			       EMPATHY_PREFS_AUTOCONNECT,
 			       &autoconnect);
 	if (autoconnect && ! no_connect &&
@@ -546,9 +558,15 @@ main (int argc, char *argv[])
 	dispatcher = empathy_dispatcher_dup_singleton ();
 	g_signal_connect (dispatcher, "dispatch", G_CALLBACK (dispatch_cb), NULL);
 
+	/* Logging */
+	log_manager = empathy_log_manager_dup_singleton ();
+	empathy_log_manager_observe (log_manager, dispatcher);
+
 	chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
 	empathy_chatroom_manager_observe (chatroom_manager, dispatcher);
 
+	ft_manager = empathy_ft_manager_dup_singleton ();
+
 	notify_init (_(PACKAGE_NAME));
 	/* Create the call factory */
 	call_factory = empathy_call_factory_initialise ();
@@ -562,8 +580,10 @@ main (int argc, char *argv[])
 	g_object_unref (mc);
 	g_object_unref (idle);
 	g_object_unref (icon);
+	g_object_unref (log_manager);
 	g_object_unref (dispatcher);
 	g_object_unref (chatroom_manager);
+	g_object_unref (ft_manager);
 
 	notify_uninit ();
 
